/*******************   main.c   ******************/

#include "global.h"

static void evalterm(int operator);
static void evalexp(void);
static void result(void);

int main(void)
  {
    int status;

    /* read expressions one at a time and evaluate them */
    while((status = readexp()) != NOMORE)
      {
        clearstack();      /* initialize stack */

        if (status == ERR) /* error in reading the expression */
            error = LONGEXP;
        else               /* evaluate the expression */
            evalexp();

        result();          /* print result */
      }

    return 0;
  }

static void evalexp(void)
  {
    int what, operator, operand;

    error = 0;

    /* obtain terms of the expression one at a time
       and do appropriate stack operations */
    while((what = next(&operator, &operand)) != NOMORE)
      {
        if (what == OPERAND)
            push(operand);
        else /* OPERATOR */
            evalterm(operator);
        if (error) return;
      }
  }

static void evalterm(int operator)
  {
    int operand1, operand2;

    operand2 = pop();
    if (error) return;
    operand1 = pop();
    if (error) return;

    switch (operator)
      {
        case '+':
            push(operand1 + operand2);
            break;
        case '*':
            push(operand1 * operand2);
            break;
        case '-':
            push(operand1 - operand2);
            break;
        case '/':
            if (operand2 != 0)
                push(operand1 / operand2);
            else
                error = ZERODIVIDE;
            break;
        default:
            error = ILLEGALOP;
            break;
      }
  }

static void result(void)
  {
    /* only the result should have been left on the stack */
    if (stacksize() > 1 && !error) error = ILLFORMEXP;

    if (error)
      {
        perror();      /* print error message */
        printexp();    /* print expression */
        printstack();  /* print stack contents */
      }
    else
        printf("%d\n", pop()); /* print result */
  }
