#include <stdio.h>
#include <stdlib.h>
#define  HEIGHT    15
#define  WIDTH     25
#define  XMIN      -1
#define  XMAX      1
#define  XINCR     0.05
#define  YMIN      0
#define  YMAX      1
#define  SYMBOL    '+'

struct point
  {
    float x, y; 
  };

struct grid
  {
    char *matrix;
    int  ht, wid;
  };

struct plotdata
  {
    struct grid frame;
    struct { float min, max; } x, y;
    char symbol;
  };

char *create(struct grid *frame);
void paint(struct plotdata *p, struct point pt);
void print(struct grid *frame);

int main(void)
  {
     struct plotdata plot =
       {
         NULL, HEIGHT, WIDTH,
         XMIN, XMAX, YMIN, YMAX,
         SYMBOL
      };
     struct point pt;

     /* create and initialize the matrix */
     if (!create(&plot.frame)) return 1;

     /* form the desired picture in this matrix */
     for (pt.x = XMIN; pt.x <= XMAX; pt.x += XINCR)
       {
         pt.y = pt.x * pt.x;
         paint(&plot, pt);
       }

     /* print it */
     print(&plot.frame);

     return 0;
  }

char *create(struct grid *frame)
  {
    int i;

    frame->matrix = (char *)
        calloc(frame->ht, frame->wid * sizeof(char));

    if (frame->matrix)  /* initialize */
        for (i = 0; i < frame->ht * frame->wid; i++)
            frame->matrix[i] = ' ';

    return frame->matrix;
  }

void paint(struct plotdata *p, struct point pt)
  {
    int i, j;

    /* normalize */
    pt.x = (pt.x - p->x.min) * p->frame.wid
           / (p->x.max - p->x.min);
    pt.y = (pt.y - p->y.min) * p->frame.ht
           / (p->y.max - p->y.min);

    i = (int)(pt.x + 0.5); /* round off */
    j = (int)(pt.y + 0.5); /* round off */

    if (i >= 0 && i < p->frame.wid &&
        j >= 0 && j < p->frame.ht)
        /* matrix[i][j] = symbol */
        p->frame.matrix[j * p->frame.wid + i]
            = p->symbol;
  }

void print(struct grid *frame)
  {
    int i, j;

    /* print from top to bottom */
    for (j = frame->ht - 1; j >= 0; j--)
      {
        for (i = 0; i < frame->wid; i++)
             /* print matrix[i][j] */
             printf("%c",
                 frame->matrix[j * frame->wid + i]);
        printf("\n");
      }
  }
